//-------------------------------------------------------------------------------------------------
// Wywietlacz alfanumeryczny ze sterownikiem HD44780
// Sterowanie w trybie 8-bitowym z odczytem flagi zajtoci
// Plik : HD44780.c	
// Mikrokontroler : Atmel AVR
// Kompilator : avr-gcc
// Autor : Radosaw Kwiecie
// rdo : http://radzio.dxp.pl/hd44780/
// Data : 24.03.2007
//-------------------------------------------------------------------------------------------------

#include "HD44780.h"


//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu bajtu do wywietacza (bez rozrnienia instrukcja/dane).
//
//-------------------------------------------------------------------------------------------------
void _LCD_Write(unsigned char dataToWrite)
{
LCD_DATA_DIR = 0xFF;

LCD_RW_PORT &= ~LCD_RW;
LCD_E_PORT |= LCD_E;
LCD_DATA_PORT = dataToWrite;
LCD_E_PORT &= ~LCD_E;
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja odczytu bajtu z wywietacza (bez rozrnienia instrukcja/dane).
//
//-------------------------------------------------------------------------------------------------

unsigned char _LCD_Read(void)
{
unsigned char tmp = 0;

LCD_DATA_DIR = 0x00;

LCD_RW_PORT |= LCD_RW;
LCD_E_PORT |= LCD_E;
asm("nop");
tmp = LCD_DATA_PIN;
LCD_E_PORT &= ~LCD_E;
return tmp;
}

//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu rozkazu do wywietlacza
//
//-------------------------------------------------------------------------------------------------
void LCD_WriteCommand(unsigned char commandToWrite)
{
LCD_RS_PORT &= ~LCD_RS;
_LCD_Write(commandToWrite);
while(LCD_ReadStatus()&0x80);
}

//-------------------------------------------------------------------------------------------------
//
// Funkcja odczytu bajtu statusowego
//
//-------------------------------------------------------------------------------------------------
unsigned char LCD_ReadStatus(void)
{
LCD_RS_PORT &= ~LCD_RS;
return _LCD_Read();
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu danych do pamici wywietlacza
//
//-------------------------------------------------------------------------------------------------
void LCD_WriteData(unsigned char dataToWrite)
{
LCD_RS_PORT |= LCD_RS;
_LCD_Write(dataToWrite);
while(LCD_ReadStatus()&0x80);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja odczytu danych z pamici wywietlacza
//
//-------------------------------------------------------------------------------------------------
unsigned char LCD_ReadData(void)
{
LCD_RS_PORT |= LCD_RS;
return _LCD_Read();
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja wywietlenia napisu na wyswietlaczu.
//
//-------------------------------------------------------------------------------------------------
void LCD_WriteText(char * text)
{
while(*text)
  LCD_WriteData(*text++);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja ustawienia wsprzdnych ekranowych
//
//-------------------------------------------------------------------------------------------------
void LCD_GoTo(unsigned char x, unsigned char y)
{
LCD_WriteCommand(HD44780_DDRAM_SET | (x + (0x40 * y)));
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja czyszczenia ekranu wywietlacza.
//
//-------------------------------------------------------------------------------------------------
void LCD_Clear(void)
{
LCD_WriteCommand(HD44780_CLEAR);
_delay_ms(2);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja przywrcenia pocztkowych wsprzdnych wywietlacza.
//
//-------------------------------------------------------------------------------------------------
void LCD_Home(void)
{
LCD_WriteCommand(HD44780_HOME);
_delay_ms(2);
}
//-------------------------------------------------------------------------------------------------
//
// Procedura inicjalizacji kontrolera HD44780.
//
//-------------------------------------------------------------------------------------------------
void LCD_Initialize(void)
{
unsigned char i;
LCD_DATA_DIR = 0xFF;
LCD_E_DIR 	|= LCD_E;   //
LCD_RS_DIR 	|= LCD_RS;  //
LCD_RW_DIR	|= LCD_RW;
_delay_ms(15); // oczekiwanie na ustalibizowanie si napiecia zasilajacego
LCD_RS_PORT &= ~LCD_RS; // wyzerowanie linii RS
LCD_E_PORT &= ~LCD_E;  // wyzerowanie linii E
LCD_RW_PORT  &= ~LCD_RW;

for(i = 0; i < 3; i++) // trzykrotne powtrzenie bloku instrukcji
  {
  LCD_E_PORT |= LCD_E;
  LCD_DATA_PORT = 0x3F;
  LCD_E_PORT &= ~LCD_E;
  _delay_ms(5); // czekaj 5ms
  }

LCD_WriteCommand(HD44780_FUNCTION_SET | HD44780_FONT5x7 | HD44780_TWO_LINE | HD44780_8_BIT); // interfejs 4-bity, 2-linie, znak 5x7
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_OFF); // wyczenie wyswietlacza
LCD_WriteCommand(HD44780_CLEAR); // czyszczenie zawartosi pamieci DDRAM
LCD_WriteCommand(HD44780_ENTRY_MODE | HD44780_EM_SHIFT_CURSOR | HD44780_EM_INCREMENT);// inkrementaja adresu i przesuwanie kursora
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_ON | HD44780_CURSOR_OFF | HD44780_CURSOR_NOBLINK); // wcz LCD, bez kursora i mrugania
}

void LCD_WriteNumber( long number, byte maxLength, byte comma, byte leadingZeros )
{
	long value = number;
	byte hunds_thousand = number / 100000;
	number -= hunds_thousand * 100000;
	byte tens_thousand = number / 10000;
	number -= tens_thousand* 10000;
	byte thousands = number / 1000;
	number -= thousands*1000;
	byte hunds = number / 100;
	number -= hunds*100;
	byte decs = number / 10;
	number -= decs*10;
	byte sings = number;
	if( maxLength >= 6 )
	{
		if( hunds_thousand == 0 && value < 100000)
			LCD_WriteData( ' ' );
		else
			LCD_WriteData( 0x30 + hunds_thousand );
	}
	if( maxLength >= 5 )
	{
		if( tens_thousand == 0 && value < 10000)
			LCD_WriteData( ' ' );
		else
			LCD_WriteData( 0x30 + tens_thousand );
	}
	if( maxLength >= 4 )
	{
		if( thousands == 0  && value < 1000 && comma < 3 && !leadingZeros  )
			LCD_WriteData( ' ' );
		else
			LCD_WriteData (0x30 + thousands );
	}
	if( maxLength >= 3 )
	{
		if( hunds == 0 && value < 100 && comma < 2 && !leadingZeros  )
			LCD_WriteData( ' ' );
		else
			LCD_WriteData ( 0x30 + hunds );
	}
	if( comma == 2 )
		LCD_WriteData( ',' );
	if( maxLength >= 2 )
	{
		if( decs == 0 && value < 10 && comma < 1 && !leadingZeros )
			LCD_WriteData( ' ' );
		else
			LCD_WriteData ( 0x30 + decs );
	}
	if( comma == 1 )
		LCD_WriteData( ',' );
	LCD_WriteData ( 0x30 + sings );	
}

//-------------------------------------------------------------------------------------------------
//
// Koniec pliku HD44780.c
//
//-------------------------------------------------------------------------------------------------
