#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "HD44780.h"
#include "rs.h"

#define NUMBER_OF_SYMBOLS_IN_BUTTON ( 50 )
#define SYMBOL_THRESHOLD  (0x0110) /* TODO BT: This should be computed accordingly */
#define TIMER_RESET_VALUE  ( 0xF000 )

// #define DEBUGGING

#define byte unsigned char

volatile byte transmission = 0;
volatile unsigned int time = 0;

volatile char c;

volatile unsigned int tab[256];
volatile byte refTab1[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */
volatile byte refTab2[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */
volatile byte refTab3[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */
volatile byte refTab4[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */
volatile byte refTab5[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */
volatile byte refTab6[NUMBER_OF_SYMBOLS_IN_BUTTON]; /* There might be more tables for more buttons */

volatile char newData;
volatile byte pwmValue, pwmCounter;
volatile byte color;

void delay ( unsigned int x )
{
	int z = 0;
	unsigned char zz = 0;
	for ( z = 0 ; z < x ; z++ ) 
		for ( zz = 0 ; zz < 255 ; zz++ )
			asm ( "NOP" );
}

ISR( USART_RXC_vect )
{
}

ISR( USART_TXC_vect )
{
}

ISR( INT0_vect )
{
	cli();

	if( !transmission )
		TCNT1 = TIMER_RESET_VALUE;
	else
	{
		time = TCNT1;
		TCNT1 = TIMER_RESET_VALUE;
	}
	
	if( transmission >= 255 )
		transmission = 254;
	transmission++;

	if( time - TIMER_RESET_VALUE < SYMBOL_THRESHOLD )	 
		tab[transmission] = 0;
	else
		tab[transmission] = 1;
	/* tab[transmission] = time - TIMER_RESET_VALUE; */

	sei();
}

ISR( TIMER1_OVF_vect )
{
	/*
	byte valueOfPD6;
	valueOfPD6 = PIND & ( 1 << 6 );
	if( valueOfPD6 )
		PORTD &= ~( 1 << 6 );
	else
		PORTD |= ( 1 << 6 ); 
	*/

	if( transmission )
	{
		tab[0] = transmission; // in tab[0] we store number of symbols
		transmission = 0;
		newData = 1;
	}
	TCNT1 = TIMER_RESET_VALUE;
}

ISR( TIMER0_OVF_vect )
{
	pwmCounter++;
	if( pwmCounter >= 100 )
		pwmCounter = 0;
	if( pwmCounter >= pwmValue )
		PORTD &= ~( color << 5 ); // Output OFF
	else
		PORTD |= ( color << 5 ); // Output ON
	TCNT0 = 0x00;
}

inline void beep() // Using actually LED, not buzzer
{
	PORTB &= ~( 1 << 6 );
	delay( 60 );
	PORTB |= ( 1 << 6 );
}

int main()
{
	rs_initialize();
	// LCD_Initialize();
	c = '*';
	newData = 0;

	// Setting up external interrupt from IR receiver
	MCUCR |= ( 1 << ISC01 );
	GICR |= ( 1 << INT0 );

	PORTD |= ( 1 << 2 ); // Turn on internal pull-up on port D.2

	TCCR1B |= ( 1 << CS11 ) | ( 1 << CS10 ); // Prescaler div 64
	TIMSK |= ( 1 << TOIE1 ); // Enable interrupt from timer 1

	TCCR0 |= ( 1 << CS00 );  // Prescaler div 1
	TIMSK |= ( 1 << TOIE0 ); // Enable interrupt from timer 0

	DDRD &= ~( 1 << 2 ); // PIND 2 as input (INT0)
	// DDRD &= ~( 1  << 3 ); // PIND 3 as input (INT1)
	DDRD |= ( 1 << 6 ); // PIND 6 as output (LED)

	/* Setup status LED */
	DDRB |= ( 1 << 6 ); // Output
	PORTB |= ( 1 << 6 ); // Turn off by default

	/* Setup programming buttons */
	DDRC &= ~( 1 << 0 );	// "plus" button
	DDRC &= ~( 1 << 1 );	// "minus" button
	DDRC &= ~( 1 << 2 );	// 
	DDRC &= ~( 1 << 3 );	// 
	DDRC &= ~( 1 << 4 );	// 
	DDRC &= ~( 1 << 5 );	// 
	PORTC |= ( 1 << 0 ) | ( 1 << 1 ) | ( 1 << 2 ) | ( 1 << 3 ) | ( 1 << 4 ) | ( 1 << 5 ); // To enable pull-ups

	/* Setup PWM output pins */
	DDRD |= ( 1 << 5 );
	DDRD |= ( 1 << 6 );
	DDRD |= ( 1 << 7 );
	PORTD &= ~( 1 << 5 ); // Off by default
	PORTD &= ~( 1 << 6 ); // Off by default
	PORTD &= ~( 1 << 7 ); // Off by default

	/* Setting default PWM value for 10% */
	pwmValue = 10;
	color = 0x01; // Something as default

	/* Reading symbols from EEPROM */
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab1[i] = eeprom_read_byte( 0*NUMBER_OF_SYMBOLS_IN_BUTTON + i );
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab2[i] = eeprom_read_byte( 1*NUMBER_OF_SYMBOLS_IN_BUTTON + i );
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab3[i] = eeprom_read_byte( 2*NUMBER_OF_SYMBOLS_IN_BUTTON + i );
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab4[i] = eeprom_read_byte( 3*NUMBER_OF_SYMBOLS_IN_BUTTON + i );
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab5[i] = eeprom_read_byte( 4*NUMBER_OF_SYMBOLS_IN_BUTTON + i );
	for( int i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
		refTab6[i] = eeprom_read_byte( 5*NUMBER_OF_SYMBOLS_IN_BUTTON + i );

	/* Sending "ready" signal though RS-232 */
	send_rs_char( 'R' );
	send_rs_char( 'D' );
	send_rs_char( 'Y' );
	send_rs_char( 0x0D );
	send_rs_char( 0x0A );

	sei();
	while(1)
	{
		// _delay_ms( 500 );
		// LCD_WriteData( c );
		if( newData )
		{
			// cli();
			beep();
			#ifdef DEBUGGING
			send_rs_char( 'R' );
			send_rs_char( ':' );
			for( int i = 0; i < 25; i++ )
			{
				send_rs_byte_hex( ( tab[i] >> 8 ) & 0xFF );
				send_rs_byte_hex( tab[i] & 0xFF );
				send_rs_char( ',' );
			}
			send_rs_char( 0x0D );
			send_rs_char( 0x0A );
			#endif

			if( !(PINC & ( 1 << 0 )) ) // programming "plus" sequence
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab1[i] = tab[i];
					eeprom_update_byte( 0*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '1' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}
			
			if( !(PINC & ( 1 << 1 )) ) // programming "minus" sequence
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab2[i] = tab[i];	
					eeprom_update_byte( 1*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '2' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}

			if( !(PINC & ( 1 << 2 )) ) // programming color +
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab3[i] = tab[i];
					eeprom_update_byte( 2*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '3' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}
			
			if( !(PINC & ( 1 << 3 )) ) // programming color -
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab4[i] = tab[i];	
					eeprom_update_byte( 3*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '4' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}
			if( !(PINC & ( 1 << 4 )) ) // programming "FULL power"
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab5[i] = tab[i];
					eeprom_update_byte( 4*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '5' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}
			
			if( !(PINC & ( 1 << 5 )) ) // programming "Green Dim"
			{
				cli();
				for( int i = 0; i < tab[0]; ++i )
				{
					refTab6[i] = tab[i];	
					eeprom_update_byte( 5*NUMBER_OF_SYMBOLS_IN_BUTTON + i, tab[i] );
				}
				send_rs_char( 'P' );
				send_rs_char( 'D' );
				send_rs_char( '6' );
				send_rs_char( 0x0D );
				send_rs_char( 0x0A );
			}

			/* Checking for match */
			{
				byte ref1Match = 1;
				byte ref2Match = 1;
				byte ref3Match = 1;
				byte ref4Match = 1;
				byte ref5Match = 1;
				byte ref6Match = 1;
				for( int i = 1; i < refTab1[0]; ++i )
					if( tab[i] != refTab1[i] )
						ref1Match = 0;
				for( int i = 1; i < refTab2[0]; ++i )
					if( tab[i] != refTab2[i] )
						ref2Match = 0;
				for( int i = 1; i < refTab3[0]; ++i )
					if( tab[i] != refTab3[i] )
						ref3Match = 0;
				for( int i = 1; i < refTab4[0]; ++i )
					if( tab[i] != refTab4[i] )
						ref4Match = 0;
				for( int i = 1; i < refTab5[0]; ++i )
					if( tab[i] != refTab5[i] )
						ref5Match = 0;
				for( int i = 1; i < refTab6[0]; ++i )
					if( tab[i] != refTab6[i] )
						ref6Match = 0;
				
				/* Reaction for match (1) */
				if( ref1Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '1' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif

					pwmValue += 10;
					if( pwmValue > 100 )
						pwmValue = 100;
				}

				/* Reaction for match (2) */
				if( ref2Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '2' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif
					if( pwmValue >= 10 )
						pwmValue -= 10;
					else
						pwmValue = 0;
				}

				/* Reaction for match (3) */
				if( ref3Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '3' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif
					PORTD &= ~( 7 << 5 ); // Output OFF
					color++;
					if( color == 8 )
						color = 0;
				}

				/* Reaction for match (4) */
				if( ref4Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '4' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif
					PORTD &= ~( 7 << 5 ); // Output OFF
					if( color == 0 )
						color = 7;
					else
						color--;
				}

				/* Reaction for match (5) */
				if( ref5Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '5' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif
					color = 7;
					pwmValue = 100;
					PORTD &= ~( 7 << 5 ); // Output OFF
				}

				/* Reaction for match (6) */
				if( ref6Match )
				{
					#ifdef DEBUGGING
					send_rs_char( 'M' );
					send_rs_char( 'T' );
					send_rs_char( '6' );
					send_rs_char( 0x0D );
					send_rs_char( 0x0A );
					#endif
					PORTD &= ~( 7 << 5 ); // Output OFF
					color = 4;
					pwmValue = 1;
				}
			}

			for( byte i = 0; i < NUMBER_OF_SYMBOLS_IN_BUTTON; ++i )
				tab[i] = 0x0000;

			newData = 0;
			sei();
		}
		delay( 5 );
	}
}

