#include "rs.h"

void rs_initialize ()
{
	const byte baud = 51; // for 9600bps @ 8MHz internal clock (PDF!)

	UBRRH = baud>>8;
	UBRRL = baud;
	UCSRB &= ~ ( 1 << RXEN );		// prevents loop-reboots
	UCSRB |= (1<<TXEN);
	UCSRC |= (1<<URSEL) | (3<<UCSZ0);

}

void send_rs_char ( unsigned char data )
{
	while ( ! ( UCSRA & ( 1 << UDRE ) ) );
	UDR = data;
}

void send_rs_byte ( unsigned char data )
{
	byte hunds = ( data ) / 100;
	byte decs = (( data - hunds*100 ) / 10);
	byte sings = data % 10;
	send_rs_char ( 0x30 + hunds );
	send_rs_char ( 0x30 + decs );
	send_rs_char ( 0x30 + sings );	
}

void send_rs_byte_hex ( unsigned char data )
{
	byte hq = data / 16;
	byte lq = data % 16;
	if ( hq < 10 )
		send_rs_char ( 0x30 + hq );
	else
		send_rs_char ( 0x41 - 10 + hq );
	if ( lq < 10 )
		send_rs_char ( 0x30 + lq );
	else
		send_rs_char ( 0x41 - 10 + lq );
}

void send_rs_message ( byte* begin )
{
	byte* pointer;
	pointer = begin;
	while ( pgm_read_byte_near ( pointer ) != 0 ) {
		send_rs_char ( pgm_read_byte_near ( pointer ) );
		pointer++;
	}
}

void send_rs_message_nl ( byte* begin )
{
	byte* pointer;
	pointer = begin;
	while ( pgm_read_byte_near ( pointer ) != 0 ) {
		send_rs_char ( pgm_read_byte_near ( pointer ) );
		pointer++;
	}
	send_rs_newline ();
}

void send_rs_hexstream ( byte* begin, int size )
{
	byte* pointer;
	pointer = begin;
	for (; pointer < begin+size; pointer++ ) {
		send_rs_byte_hex ( *pointer );
	}
}

void send_rs_newline ()
{
	send_rs_char ( 0x0A );
	send_rs_char ( 0x0D );
}
